package com.zoniot.ccrc.commom.utils;

import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * 导出Excel公共方法
 */
public class ExcelUtil {

    //显示的导出表的标题
    private String title;
    //导出表的列名
    private String[] rowName;

    private List<Object[]> dataList = new ArrayList<Object[]>();

    private List<String> eList = new ArrayList<String>();

    HttpServletResponse response;

    //构造方法，传入要导出的数据
    public ExcelUtil(String title, String[] rowName, List<Object[]> dataList) {
        this.dataList = dataList;
        this.rowName = rowName;
        this.title = title;
    }
    public ExcelUtil(String title, String[] rowName, List<Object[]> dataList, List<String>list) {
        this.dataList = dataList;
        this.rowName = rowName;
        this.title = title;
        this.eList = list;
    }

    /*
     * 导出数据
     * */
    public void export(HttpServletResponse response) throws Exception {
        try {
            HSSFWorkbook workbook = new HSSFWorkbook();                        // 创建工作簿对象
            int pointsDataLimit = 60000;
            int part = dataList.size() / pointsDataLimit;//分批数
            for (int p = 0; p <= part; p++) {
                if(dataList.size() <  pointsDataLimit){
                    pointsDataLimit = dataList.size();
                }
                List<Object[]> listPage = dataList.subList(0, pointsDataLimit);
                HSSFSheet sheet = workbook.createSheet("Sheet"+(p+1));                     // 创建工作表

                // 产生表格标题行
                HSSFRow rowm = sheet.createRow(0);
                HSSFCell cellTiltle = rowm.createCell(0);

                //sheet样式定义【getColumnTopStyle()/getStyle()均为自定义方法 - 在下面  - 可扩展】
                HSSFCellStyle columnTopStyle = this.getColumnTopStyle(workbook);//获取列头样式对象
                HSSFCellStyle style = this.getStyle(workbook);                    //单元格样式对象

                sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, (rowName.length - 1)));
                cellTiltle.setCellStyle(columnTopStyle);
                cellTiltle.setCellValue(title);


                HSSFRow rowRowName = null;
                if(eList != null && eList.size() > 0){
                    for(int i=0; i<eList.size(); i++){
                        Row searchFormRow = sheet.createRow(2+i);
                        Cell searchFormCell = searchFormRow.createCell(0);
                        searchFormCell.setCellValue(eList.get(i));
                        sheet.addMergedRegion(new CellRangeAddress(searchFormRow.getRowNum(),
                                searchFormRow.getRowNum(), 0, (rowName.length - 1)));
                    }
                    rowRowName = sheet.createRow(4);                // 在索引4的位置创建行(最顶端的行开始的第二行)
                }else{
                    rowRowName = sheet.createRow(2);                // 在索引2的位置创建行(最顶端的行开始的第二行)
                }

                // 定义所需列数
                int columnNum = rowName.length;

                // 将列头设置到sheet的单元格中
                for (int n = 0; n < columnNum; n++) {
                    HSSFCell cellRowName = rowRowName.createCell(n);                //创建列头对应个数的单元格
                    cellRowName.setCellType(CellType.STRING);                //设置列头单元格的数据类型
                    HSSFRichTextString text = new HSSFRichTextString(rowName[n]);
                    cellRowName.setCellValue(text);                                    //设置列头单元格的值
                    cellRowName.setCellStyle(columnTopStyle);                        //设置列头单元格样式
                }

                //将查询出的数据设置到sheet对应的单元格中
                for (int i = 0; i < listPage.size(); i++) {
                    Object[] obj = listPage.get(i);//遍历每个对象
                    HSSFRow row = null;
                    if(eList != null && eList.size() > 0){
                        row = sheet.createRow(i + 5);//创建所需的行数
                    }else{
                        row = sheet.createRow(i + 3);//创建所需的行数
                    }


                    for (int j = 0; j < obj.length; j++) {
                        HSSFCell cell = null;   //设置单元格的数据类型
                        if (j == 0) {
                            cell = row.createCell(j, CellType.NUMERIC);
                            cell.setCellValue(i + 1);
                        } else {
                            cell = row.createCell(j, CellType.STRING);
                            if (obj[j] != null && !"".equals(obj[j])) {
                                cell.setCellValue(obj[j].toString());                        //设置单元格的值
                            }else {
                                cell.setCellValue("-");
                            }
                        }
                        cell.setCellStyle(style);                                    //设置单元格样式
                    }
                }
                //让列宽随着导出的列长自动适应
                for (int colNum = 0; colNum < columnNum; colNum++) {
                    int columnWidth = sheet.getColumnWidth(colNum) / 256;
                    for (int rowNum = 0; rowNum <= sheet.getLastRowNum(); rowNum++) {
                        HSSFRow currentRow;
                        //当前行未被使用过
                        if (sheet.getRow(rowNum) == null) {
                            currentRow = sheet.createRow(rowNum);
                        } else {
                            currentRow = sheet.getRow(rowNum);
                        }
                        if (currentRow.getCell(colNum) != null) {
                            HSSFCell currentCell = currentRow.getCell(colNum);
                            String numbers = "";

                            if (currentCell.getCellTypeEnum() == CellType.STRING) {
                                int length = calculateLength(currentCell.getStringCellValue());
                                if (columnWidth < length) {
                                    columnWidth = length;
                                }
                                if(eList != null && eList.size() > 0){
                                    if(rowNum == 4){
                                        numbers = currentCell != null ? currentCell.getStringCellValue() : "";
                                    }
                                    if(numbers.equals("序号")){
                                        columnWidth = 18;
                                    }
                                }
                            }
                        }
                    }
                    if(columnWidth > 60){
                        sheet.setColumnWidth(colNum, 60 * 256);
                    }else {
                        sheet.setColumnWidth(colNum, columnWidth * 256);
                    }
                }
                //剔除
                dataList.subList(0, pointsDataLimit).clear();
            }

            OutputStream out = null;
            try {
                //输出数据流
                DateTimeFormatter f = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
                LocalDateTime date = LocalDateTime.now();
                String fileName = title + "-" + date.format(f) + ".xls";

                String headStr = "attachment;filename*=UTF-8''" + URLEncoder.encode(fileName, "UTF-8");

                response.setContentType("APPLICATION/OCTET-STREAM");
                response.setHeader("Content-Disposition", headStr);
                out = response.getOutputStream();
                workbook.write(out);

                //输出Excel文件
                /*FileOutputStream output = new FileOutputStream("D:\\test.xls");
                workbook.write(output);
                output.flush();
                workbook.close();*/
            } catch (IOException e) {
                e.printStackTrace();
            }finally {
                workbook.close();
                if (out != null) {
                    out.close();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /*
     * 列头单元格样式
     */
    public HSSFCellStyle getColumnTopStyle(HSSFWorkbook workbook) {

        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short) 11);
        //字体加粗
        font.setBold(true);
        //设置字体名字
        font.setFontName("Courier New");
        //设置样式;
        HSSFCellStyle style = workbook.createCellStyle();
        //设置底边框;
        style.setBorderBottom(BorderStyle.THIN);
        //设置底边框颜色;
        style.setBottomBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        //设置左边框;
        style.setBorderLeft(BorderStyle.THIN);
        //设置左边框颜色;
        style.setLeftBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        //设置右边框;
        style.setBorderRight(BorderStyle.THIN);
        //设置右边框颜色;
        style.setRightBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        //设置顶边框;
        style.setBorderTop(BorderStyle.THIN);
        //设置顶边框颜色;
        style.setTopBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(false);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HorizontalAlignment.CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(VerticalAlignment.CENTER);

        return style;

    }

    /*
   * 列数据信息单元格样式
   */
    public HSSFCellStyle getStyle(HSSFWorkbook workbook) {
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        //font.setFontHeightInPoints((short)10);
        //字体加粗
        //font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        //设置字体名字
        font.setFontName("Courier New");
        //设置样式;
        HSSFCellStyle style = workbook.createCellStyle();
        //设置底边框;
        style.setBorderBottom(BorderStyle.THIN);
        //设置底边框颜色;
        style.setBottomBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        //设置左边框;
        style.setBorderLeft(BorderStyle.THIN);
        //设置左边框颜色;
        style.setLeftBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        //设置右边框;
        style.setBorderRight(BorderStyle.THIN);
        //设置右边框颜色;
        style.setRightBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        //设置顶边框;
        style.setBorderTop(BorderStyle.THIN);
        //设置顶边框颜色;
        style.setTopBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(false);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HorizontalAlignment.CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(VerticalAlignment.CENTER);

        style.setLocked(false);

        return style;

    }

    public static int calculateLength(String str){
        int m = 0;
        char arr[] = str.toCharArray();
        for (int i = 0; i < arr.length; i++) {
            char c = arr[i];
            // 中文字符(根据Unicode范围判断),中文字符长度为3
            if ((c >= 0x0391 && c <= 0xFFE5)) {
                m = m + 3;
            } else
            {
                m = m + 2;
            }
        }
        return m;
    }

    public static void main(String[] args) throws Exception {
        String title = "测试一览表";
        String[] rowsName = new String[]{"序号", "名称"};
        List<Object[]> dataList = new ArrayList<Object[]>();
        List<String> list = new ArrayList<>();
        list.add("004a770124007922");
        list.add("004a770124007922");
        list.add("004a770124007922");
        /*list.add("大厅");
        list.add(null);
        list.add("1");
        list.add("大厅");
        list.add("B2");
        list.add("B2");
        list.add("微型消防站消防栓门");*/
        Object[] objs = null;
        for (int i = 0; i < list.size(); i++) {
            String str = list.get(i);
            objs = new Object[rowsName.length];
            objs[0] = i;
            objs[1] = str;
            dataList.add(objs);
        }
        ExcelUtil ex = new ExcelUtil(title, rowsName, dataList);
        ex.export(null);
    }

}
