package com.zoniot.ccrc.controller.common;


import com.zoniot.ccrc.commom.exception.ServiceException;
import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.commom.utils.ByteArrayUtils;
import com.zoniot.ccrc.commom.utils.RedisUtil;
import com.zoniot.ccrc.commom.utils.VerifyCodeUtil;
import com.zoniot.ccrc.dto.ValidateCode;
import com.zoniot.ccrc.service.SmsAliyunService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
@Api(tags = "短信")
@RequestMapping("sms")
public class SmsController {

    @Autowired
    private SmsAliyunService smsService;
    @Autowired
    private RedisUtil redisUtil;
    @Value("${spring.profiles.active}")
    private String active;

    @PostMapping("/send")
    @ResponseBody
    @ApiOperation(value = "发送短信")
    public AjaxMessage getCode(
            @ApiParam(value = "手机号码", required = true) @RequestParam(required = true) String phone
    ) {
        if (StringUtils.equals(phone, "18800000000") || StringUtils.equals(phone, "18800000001")) {
            return new AjaxMessage(ResultStatus.OK);
        }
        String key = "smsValidTime:" + phone;

        byte[] redisValidateCodeByte = redisUtil.get(key.getBytes());
        if (redisValidateCodeByte != null) {
            throw new ServiceException(-900,"60秒内无法发送");
        }
        String verifyCode = VerifyCodeUtil.generateTextCode(VerifyCodeUtil.TYPE_NUM_ONLY, 4, null);
        //保存redis
        String smsKey = "smsCode:" + phone;
        redisUtil.setExpire(smsKey.getBytes(), ByteArrayUtils.objectToBytes(verifyCode).get(), 900);//15分钟过期
        smsService.sendVerificationCodeSms(phone, verifyCode);
        redisUtil.setExpire(key.getBytes(), "".getBytes(), 60);//60秒

        return new AjaxMessage(ResultStatus.OK);
    }


    @PostMapping("/v2/send")
    @ResponseBody
    @ApiOperation(value = "发送短信")
    public AjaxMessage getCode(
            @ApiParam(value = "手机号码", required = true) @RequestParam(required = true) String mobile,
            @ApiParam(value = "验证码", required = true) @RequestParam(required = true) String code,
            @ApiParam(value = "随机数 uuid", required = true) @RequestParam String random
    ) {
        if(StringUtils.equals("sit",active)){
            return new AjaxMessage(ResultStatus.OK);
        }
        if (StringUtils.equals(mobile, "18800000000") || StringUtils.equals(mobile, "18800000001")) {
            return new AjaxMessage(ResultStatus.OK);
        }
        String validKey = "smsValidTime:" + mobile;

        byte[] redisValidateCodeByteValid = redisUtil.get(validKey.getBytes());
        if (redisValidateCodeByteValid != null) {
            throw new ServiceException(-900,"60秒内无法发送");
        }


        String key = "validateCode:" + random;
        byte[] redisValidateCodeByte = redisUtil.get(key.getBytes());
        if (redisValidateCodeByte == null) {
            throw new ServiceException(-101,"验证码已经过期");
        } else {
            ValidateCode validateCode = (ValidateCode) ByteArrayUtils.bytesToObject(redisValidateCodeByte).get();
            if (validateCode.isExpried() || !StringUtils.equals(validateCode.getCode(), code)) {
                throw new ServiceException(-101,"验证码错误");
            }
        }
        String verifyCode = VerifyCodeUtil.generateTextCode(VerifyCodeUtil.TYPE_NUM_ONLY, 4, null);
        //保存redis
        String smsKey = "smsCode:" + mobile;
        redisUtil.setExpire(smsKey.getBytes(), ByteArrayUtils.objectToBytes(verifyCode).get(), 900);//15分钟过期
        smsService.sendVerificationCodeSms(mobile, verifyCode);
        redisUtil.setExpire(validKey.getBytes(), "".getBytes(), 60);//60秒

        //清除验证码
        redisUtil.del(key.getBytes());
        return new AjaxMessage(ResultStatus.OK);
    }
}
