package com.zoniot.ccrc.security.openid;


import com.zoniot.ccrc.service.WechatService;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.util.Assert;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class OpenidLoginAuthenticationFilter extends AbstractAuthenticationProcessingFilter {
    // ~ Static fields/initializers
    // =====================================================================================

    public static final String SPRING_SECURITY_FORM_CODE_KEY = "code";
    public static final String SPRING_SECURITY_FORM_CUSTOMER_KEY = "customerId";

    private String codeParameter = SPRING_SECURITY_FORM_CODE_KEY;
    private String customerParameter = SPRING_SECURITY_FORM_CUSTOMER_KEY;
    private boolean postOnly = true;

    // ~ Constructors
    // ===================================================================================================

    public OpenidLoginAuthenticationFilter() {
        super(new AntPathRequestMatcher("/user/openidLogin", "POST"));
    }

    // ~ Methods
    // ========================================================================================================

    @Override
    public Authentication attemptAuthentication(HttpServletRequest request,
                                                HttpServletResponse response) throws AuthenticationException {
        if (postOnly && !request.getMethod().equals("POST")) {
            throw new AuthenticationServiceException(
                    "Authentication method not supported: " + request.getMethod());
        }

        String code = obtainCode(request);
        String customerId = obtainCustomerId(request);

        if (code == null) {
            code = "";
        }

        String openid = WechatService.getOpenid(code,customerId);


        OpenidLoginAuthenticationToken authRequest = new OpenidLoginAuthenticationToken(openid);

        // Allow subclasses to set the "details" property
        setDetails(request, authRequest);

        return this.getAuthenticationManager().authenticate(authRequest);
    }

    private String obtainCode(HttpServletRequest request) {
        return request.getParameter(codeParameter);
    }
    private String obtainCustomerId(HttpServletRequest request) {
        return request.getParameter(customerParameter);
    }



    protected void setDetails(HttpServletRequest request, OpenidLoginAuthenticationToken authRequest) {
        authRequest.setDetails(authenticationDetailsSource.buildDetails(request));
    }


    public void setCodeParameter(String codeParameter) {
        Assert.hasText(codeParameter, "code parameter must not be empty or null");
        this.codeParameter = codeParameter;
    }



    public void setPostOnly(boolean postOnly) {
        this.postOnly = postOnly;
    }

    public final String getCodeParameter() {
        return codeParameter;
    }

}
