package com.zoniot.ccrc.security.openid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.SecurityConfigurerAdapter;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.web.DefaultSecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.stereotype.Component;

@Component
public class OpenidSecurityConfig extends SecurityConfigurerAdapter<DefaultSecurityFilterChain, HttpSecurity> {
    @Autowired
    @Qualifier("userAuthenticationSuccessHandler")
    private AuthenticationSuccessHandler authenticationSuccessHandler;
    @Autowired
    @Qualifier("userAuthenticationFailureHandler")
    private AuthenticationFailureHandler authenticationFailureHandler;
    @Autowired
    private OpenidLoginUserDetailService userDetailsService;
    //@Autowired
   // private  OpenidLoginAuthenticationFilter openidLoginAuthenticationFilter;


    @Override
    public void configure(HttpSecurity http) throws Exception {
        OpenidLoginAuthenticationFilter openidLoginAuthenticationFilter = new OpenidLoginAuthenticationFilter();
        openidLoginAuthenticationFilter.setAuthenticationManager(http.getSharedObject(AuthenticationManager.class));
        openidLoginAuthenticationFilter.setAuthenticationSuccessHandler(authenticationSuccessHandler);
        openidLoginAuthenticationFilter.setAuthenticationFailureHandler(authenticationFailureHandler);

        OpenidLoginAuthenticationProvider openidLoginAuthenticationProvider = new OpenidLoginAuthenticationProvider();
        openidLoginAuthenticationProvider.setUserDetailsService(userDetailsService);
        http.authenticationProvider(openidLoginAuthenticationProvider)
                .addFilterAfter(openidLoginAuthenticationFilter, UsernamePasswordAuthenticationFilter.class);
    }
}
