package com.zoniot.ccrc.service;

import com.alibaba.fastjson.JSONObject;
import com.zoniot.ccrc.commom.utils.HttpRequest;
import com.zoniot.ccrc.commom.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;

@Slf4j
@Service
public class AccessTokenService {
    @Autowired
    private RedisUtil redisUtil;


    @Value("${wechat.mp.appid}")
    private String mpAppid;
    @Value("${wechat.mp.app-secret}")
    private String mpAppSecret;

    /**
     * 获取微信小程序accessToken
     * @return
     */
    public String getMpAccessToken(){
        String token = null;
        token = redisUtil.get("ccrcMpAccessToken");
        if (token != null) {
            return token;
        }else {
            String tokenUrl = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" +
                    mpAppid + "&secret=" + mpAppSecret;
            try {
                String res = HttpRequest.doGet(tokenUrl);
                JSONObject jsonObj = (JSONObject) JSONObject.parse(res);
                log.info(jsonObj.toJSONString());
                String accessTokenTem = jsonObj.getString("access_token");
                if (accessTokenTem != null && !"".equals(accessTokenTem)) {
                    redisUtil.setExpire("ccrcMpAccessToken",accessTokenTem,300);
                    return accessTokenTem;
                } else {
                    log.info("请求token错误");
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return token;
    }
}
