package com.zoniot.ccrc.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import sun.misc.BASE64Encoder;

import static com.google.common.collect.Lists.newArrayList;

@Slf4j
@Component
public class IotService {
    private RestTemplate restTemplate = new RestTemplate();

    @Value("${iot.url}")
    private String iotUrl;

    public String getLoginIotToken(String account){
        log.info("begin IotService login account={}",account);
        String url = iotUrl + "/api/integration/auth/login";

        MultiValueMap<String, String> map= new LinkedMultiValueMap<String, String>();
        map.add("source","ccrc");
        map.add("code",new BASE64Encoder().encode(account.getBytes()));
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(map,headers);
        ResponseEntity<JSONObject> responseEntity  = restTemplate.postForEntity(url, request, JSONObject.class);
        log.info("responseEntity ={}", JSON.toJSONString(responseEntity));
        if(responseEntity !=null && responseEntity.getStatusCode() == HttpStatus.OK){
            JSONObject jsonObject =  responseEntity.getBody();
            if(jsonObject != null){
                if(jsonObject.getInteger("status")==0){
                    String token = jsonObject.getString("data");
                    log.info("IotService getLoginIotToken success res={}",token);
                    return token;
                }else {
                    log.info("IotService getLoginIotToken fail");
                    return null;

                }
            }else {
                return null;
            }
        }

        return null;
    }

    public void logout(String iotToken){
        String url = iotUrl + "/api/user/logout";

        HttpHeaders headers = new HttpHeaders();
        headers.set("Authorization","Bearer " + iotToken);
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(headers);
        ResponseEntity<JSONObject> responseEntity  = restTemplate.postForEntity(url, request, JSONObject.class);
        log.info("responseEntity ={}", JSON.toJSONString(responseEntity));
        if(responseEntity !=null && responseEntity.getStatusCode() == HttpStatus.OK){
            JSONObject jsonObject =  responseEntity.getBody();
            if(jsonObject != null){
                if(jsonObject.getInteger("status")==0){
                    log.info("IotService logout success");
                }else {
                    log.info("IotService logout fail");
                }
            }
        }
    }

}
